package org.gcube.informationsystem.utils.documentation.model.properties;

import java.util.HashSet;
import java.util.Set;

import org.gcube.informationsystem.base.reference.properties.PropertyElement;
import org.gcube.informationsystem.model.reference.properties.Property;
import org.gcube.informationsystem.types.TypeMapper;
import org.gcube.informationsystem.types.reference.properties.PropertyDefinition;
import org.gcube.informationsystem.utils.documentation.model.DocumentationGenerator;
import org.gcube.informationsystem.utils.documentation.rst.table.Cell;
import org.gcube.informationsystem.utils.documentation.rst.table.Row;
import org.gcube.informationsystem.utils.documentation.rst.table.RowType;
import org.gcube.informationsystem.utils.documentation.rst.table.Table;

/**
 * @author Luca Frosini (ISTI - CNR)
 */
public class PropertyDocumentationGenerator<P extends Property> extends DocumentationGenerator<P> {
	
	public PropertyDocumentationGenerator(Class<P> clz) {
		super(clz, TypeMapper.getType(PropertyElement.class));
	}
	
	protected Row getHeadingRow() {
		Row row = new Row(RowType.HEADING);
		Cell name = new Cell();
		name.setText("Name");
		row.appendCell(name);
		Cell type = new Cell();
		type.setText("Type");
		row.appendCell(type);
		Cell attributes = new Cell();
		attributes.setText("Attributes");
		row.appendCell(attributes);
		Cell description = new Cell();
		description.setText("Description");
		row.appendCell(description);
		return row;
	}
	
	protected String getPropertyAttributes(PropertyDefinition propertyDefinition) {
		StringBuffer stringBuffer = new StringBuffer();
		
		boolean mandatory = propertyDefinition.isMandatory();
		stringBuffer.append("``Mandatory:");
		stringBuffer.append(mandatory);
		
		boolean readOnly = propertyDefinition.isReadonly();
		stringBuffer.append("`` ``ReadOnly:");
		stringBuffer.append(readOnly);
		
		boolean notNull = propertyDefinition.isNotnull();
		stringBuffer.append("`` ``NotNull:");
		stringBuffer.append(notNull);
		
		String regex = propertyDefinition.getRegexp();
		if(regex!=null && regex.compareTo("")!=0) {
			stringBuffer.append("`` ``Regex:");
			stringBuffer.append(regex);
		}
		
		Integer max = propertyDefinition.getMax();
		if(max!=null) {
			stringBuffer.append("`` ``Max:");
			stringBuffer.append(max);
		}
		
		Integer min = propertyDefinition.getMin();
		if(min!=null) {
			stringBuffer.append("`` ``Min:");
			stringBuffer.append(min);
		}
		stringBuffer.append("``");
		return stringBuffer.toString();
	}
	
	protected Row getPropertyRow(PropertyDefinition propertyDefinition) {
		Row row = new Row(RowType.NORMAL);
		Cell name = new Cell();
		name.setText(propertyDefinition.getName());
		row.appendCell(name);
		Cell type = new Cell();
		type.setText(propertyDefinition.getType());
		row.appendCell(type);
		Cell attributes = new Cell();
		attributes.setText(getPropertyAttributes(propertyDefinition));
		row.appendCell(attributes);
		Cell description = new Cell();
		description.setText(propertyDefinition.getDescription());
		row.appendCell(description);
		return row;
	}
	
	@Override
	protected Table getTable() {
		
		Table table = new Table();
		table.appendRow(getHeadingRow());
		Set<PropertyDefinition> properties = type.getProperties();
		if(properties!=null && properties.size()>0) {
			Set<PropertyDefinition> notMandatoryProperties = new HashSet<>();
			for(PropertyDefinition propertyDefinition : properties) {
				if(propertyDefinition.isMandatory()) {
					/*
					 * Adding Mandatory properties first in the table
					 */
					table.appendRow(getPropertyRow(propertyDefinition));
				}else {
					notMandatoryProperties.add(propertyDefinition);
				}
			}
			for(PropertyDefinition propertyDefinition : notMandatoryProperties) {
				table.appendRow(getPropertyRow(propertyDefinition));
			}
		}else {
			table.appendRow(getNoPropertyRow());
		}
		return table;
	}
	
	
	protected Row getNoPropertyRow() {
		Row row = new Row(RowType.NORMAL);
		Cell noProp = new Cell();
		noProp.setText("This type does not define any additional attributes.");
		row.appendCell(noProp);
		row.appendCell(noProp);
		row.appendCell(noProp);
		row.appendCell(noProp);
		return row;
	}

}
